@@ -23,17 +23,18 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_CALENDAR_H__
 #define __GTK_CALENDAR_H__
 
-#include <gdk/gdk.h>
-#include <gtk/gtkwidget.h>
 
-/* Not needed, retained for compatibility -Yosh */
-#include <gtk/gtksignal.h>
+#include <gtk/gtkwidget.h>
 
 
 G_BEGIN_DECLS
@@ -49,45 +50,85 @@ G_BEGIN_DECLS
 typedef struct _GtkCalendar	       GtkCalendar;
 typedef struct _GtkCalendarClass       GtkCalendarClass;
 
+typedef struct _GtkCalendarPrivate     GtkCalendarPrivate;
+
+/**
+ * GtkCalendarDisplayOptions:
+ * @GTK_CALENDAR_SHOW_HEADING: Specifies that the month and year should be displayed.
+ * @GTK_CALENDAR_SHOW_DAY_NAMES: Specifies that three letter day descriptions should be present.
+ * @GTK_CALENDAR_NO_MONTH_CHANGE: Prevents the user from switching months with the calendar.
+ * @GTK_CALENDAR_SHOW_WEEK_NUMBERS: Displays each week numbers of the current year, down the
+ * left side of the calendar.
+ * @GTK_CALENDAR_WEEK_START_MONDAY: Since GTK+ 2.4, this option is deprecated and ignored by GTK+.
+ * The information on which day the calendar week starts is derived from the locale.
+ * @GTK_CALENDAR_SHOW_DETAILS: Just show an indicator, not the full details
+ * text when details are provided. See gtk_calendar_set_detail_func().
+ *
+ * These options can be used to influence the display and behaviour of a #GtkCalendar.
+ */
 typedef enum
 {
   GTK_CALENDAR_SHOW_HEADING		= 1 << 0,
   GTK_CALENDAR_SHOW_DAY_NAMES		= 1 << 1,
   GTK_CALENDAR_NO_MONTH_CHANGE		= 1 << 2,
   GTK_CALENDAR_SHOW_WEEK_NUMBERS	= 1 << 3,
-  GTK_CALENDAR_WEEK_START_MONDAY	= 1 << 4
+  GTK_CALENDAR_WEEK_START_MONDAY	= 1 << 4,
+  GTK_CALENDAR_SHOW_DETAILS		= 1 << 5
 } GtkCalendarDisplayOptions;
 
+/**
+ * GtkCalendarDetailFunc:
+ * @calendar: a #GtkCalendar.
+ * @year: the year for which details are needed.
+ * @month: the month for which details are needed.
+ * @day: the day of @month for which details are needed.
+ * @user_data: the data passed with gtk_calendar_set_detail_func().
+ *
+ * This kind of functions provide Pango markup with detail information for the
+ * specified day. Examples for such details are holidays or appointments. The
+ * function returns %NULL when no information is available.
+ *
+ * Since: 2.14
+ *
+ * Return value: Newly allocated string with Pango markup with details
+ * for the specified day, or %NULL.
+ */
+typedef gchar* (*GtkCalendarDetailFunc) (GtkCalendar *calendar,
+                                         guint        year,
+                                         guint        month,
+                                         guint        day,
+                                         gpointer     user_data);
+
 struct _GtkCalendar
 {
   GtkWidget widget;
   
-  GtkStyle  *header_style;
-  GtkStyle  *label_style;
+  GtkStyle  *GSEAL (header_style);
+  GtkStyle  *GSEAL (label_style);
   
-  gint month;
-  gint year;
-  gint selected_day;
+  gint GSEAL (month);
+  gint GSEAL (year);
+  gint GSEAL (selected_day);
   
-  gint day_month[6][7];
-  gint day[6][7];
+  gint GSEAL (day_month[6][7]);
+  gint GSEAL (day[6][7]);
   
-  gint num_marked_dates;
-  gint marked_date[31];
-  GtkCalendarDisplayOptions  display_flags;
-  GdkColor marked_date_color[31];
+  gint GSEAL (num_marked_dates);
+  gint GSEAL (marked_date[31]);
+  GtkCalendarDisplayOptions  GSEAL (display_flags);
+  GdkColor GSEAL (marked_date_color[31]);
   
-  GdkGC *gc;
-  GdkGC *xor_gc;
+  GdkGC *GSEAL (gc);			/* unused */
+  GdkGC *GSEAL (xor_gc);		/* unused */
 
-  gint focus_row;
-  gint focus_col;
+  gint GSEAL (focus_row);
+  gint GSEAL (focus_col);
 
-  gint highlight_row;
-  gint highlight_col;
+  gint GSEAL (highlight_row);
+  gint GSEAL (highlight_col);
   
-  gpointer private_data;
-  gchar grow_space [32];
+  GtkCalendarPrivate *GSEAL (priv);
+  gchar GSEAL (grow_space [32]);
 
   /* Padding for future expansion */
   void (*_gtk_reserved1) (void);
@@ -132,18 +173,23 @@ void	   gtk_calendar_set_display_options (GtkCalendar    	      *calendar,
 					     GtkCalendarDisplayOptions flags);
 GtkCalendarDisplayOptions
            gtk_calendar_get_display_options (GtkCalendar   	      *calendar);
-#ifndef GTK_DISABLE_DEPRECATED
-void	   gtk_calendar_display_options (GtkCalendar		  *calendar,
-					 GtkCalendarDisplayOptions flags);
-#endif
-
 void	   gtk_calendar_get_date	(GtkCalendar *calendar, 
 					 guint	     *year,
 					 guint	     *month,
 					 guint	     *day);
-void	   gtk_calendar_freeze		(GtkCalendar *calendar);
-void	   gtk_calendar_thaw		(GtkCalendar *calendar);
 
+void       gtk_calendar_set_detail_func (GtkCalendar           *calendar,
+                                         GtkCalendarDetailFunc  func,
+                                         gpointer               data,
+                                         GDestroyNotify         destroy);
+
+void       gtk_calendar_set_detail_width_chars (GtkCalendar    *calendar,
+                                                gint            chars);
+void       gtk_calendar_set_detail_height_rows (GtkCalendar    *calendar,
+                                                gint            rows);
+
+gint       gtk_calendar_get_detail_width_chars (GtkCalendar    *calendar);
+gint       gtk_calendar_get_detail_height_rows (GtkCalendar    *calendar);
 
 G_END_DECLS
 
