@@ -1057,12 +1057,6 @@ gtk_container_destroy (GtkObject *object)
   if (GTK_CONTAINER_RESIZE_PENDING (container))
     _gtk_container_dequeue_resize_handler (container);
 
-  if (container->focus_child)
-    {
-      g_object_unref (container->focus_child);
-      container->focus_child = NULL;
-    }
-
   /* do this before walking child widgets, to avoid
    * removing children from focus chain one by one.
    */
@@ -1376,6 +1370,9 @@ _gtk_container_queue_resize (GtkContainer *container)
     {
       GTK_PRIVATE_SET_FLAG (widget, GTK_ALLOC_NEEDED);
       GTK_PRIVATE_SET_FLAG (widget, GTK_REQUEST_NEEDED);
+      GTK_PRIVATE_SET_FLAG (widget, GTK_WIDTH_REQUEST_NEEDED);
+      GTK_PRIVATE_SET_FLAG (widget, GTK_HEIGHT_REQUEST_NEEDED);
+
       if ((resize_container && widget == GTK_WIDGET (resize_container)) ||
 	  !widget->parent)
 	break;
@@ -1541,55 +1538,6 @@ struct _GtkForeachData
   gpointer           callback_data;
 };
 
-static void
-gtk_container_foreach_unmarshal (GtkWidget *child,
-				 gpointer data)
-{
-  GtkForeachData *fdata = (GtkForeachData*) data;
-  GtkArg args[2];
-  
-  /* first argument */
-  args[0].name = NULL;
-  args[0].type = G_TYPE_FROM_INSTANCE (child);
-  GTK_VALUE_OBJECT (args[0]) = GTK_OBJECT (child);
-  
-  /* location for return value */
-  args[1].name = NULL;
-  args[1].type = G_TYPE_NONE;
-  
-  fdata->callback (fdata->container, fdata->callback_data, 1, args);
-}
-
-void
-gtk_container_foreach_full (GtkContainer       *container,
-			    GtkCallback         callback,
-			    GtkCallbackMarshal  marshal,
-			    gpointer            callback_data,
-			    GDestroyNotify      notify)
-{
-  g_return_if_fail (GTK_IS_CONTAINER (container));
-
-  if (marshal)
-    {
-      GtkForeachData fdata;
-  
-      fdata.container     = GTK_OBJECT (container);
-      fdata.callback      = marshal;
-      fdata.callback_data = callback_data;
-
-      gtk_container_foreach (container, gtk_container_foreach_unmarshal, &fdata);
-    }
-  else
-    {
-      g_return_if_fail (callback != NULL);
-
-      gtk_container_foreach (container, callback, &callback_data);
-    }
-
-  if (notify)
-    notify (callback_data);
-}
-
 /**
  * gtk_container_set_focus_child:
  * @container: a #GtkContainer
@@ -1600,9 +1548,6 @@ gtk_container_foreach_full (GtkContainer       *container,
  * This function emits the GtkContainer::set_focus_child signal of
  * @container. Implementations of #GtkContainer can override the
  * default behaviour by overriding the class closure of this signal.
- *
- * This is function is mostly meant to be used by widgets. Applications can use
- * gtk_widget_grab_focus() to manualy set the focus to a specific widget.
  */
 void
 gtk_container_set_focus_child (GtkContainer *container,
@@ -1619,12 +1564,10 @@ gtk_container_set_focus_child (GtkContainer *container,
  * gtk_container_get_focus_child:
  * @container: a #GtkContainer
  *
- * Returns the current focus child widget inside @container. This is not the
- * currently focused widget. That can be obtained by calling
- * gtk_window_get_focus().
+ * Returns the current focus child widget inside @container.
  *
- * Returns: The child widget which will recieve the focus inside @container when
- *          the @conatiner is focussed, or %NULL if none is set.
+ * Returns: The child widget which has the focus
+ *          inside @container, or %NULL if none is set.
  *
  * Since: 2.14
  **/
@@ -2359,10 +2302,9 @@ chain_widget_destroyed (GtkWidget *widget,
 }
 
 /**
- * gtk_container_set_focus_chain:
+ * gtk_container_set_focus_chain: 
  * @container: a #GtkContainer
- * @focusable_widgets: (transfer none) (element-type GtkWidget):
- *     the new focus chain
+ * @focusable_widgets: the new focus chain
  *
  * Sets a focus chain, overriding the one computed automatically by GTK+.
  * 
