@@ -21,16 +21,20 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_RADIO_MENU_ITEM_H__
 #define __GTK_RADIO_MENU_ITEM_H__
 
 
-#include <gdk/gdk.h>
 #include <gtk/gtkcheckmenuitem.h>
 
+
 G_BEGIN_DECLS
 
 #define GTK_TYPE_RADIO_MENU_ITEM	      (gtk_radio_menu_item_get_type ())
@@ -47,14 +51,14 @@ typedef struct _GtkRadioMenuItemClass  GtkRadioMenuItemClass;
 struct _GtkRadioMenuItem
 {
   GtkCheckMenuItem check_menu_item;
-  
-  GSList *group;
+
+  GSList *GSEAL (group);
 };
 
 struct _GtkRadioMenuItemClass
 {
   GtkCheckMenuItemClass parent_class;
-  
+
   /* Signals */
   void (*group_changed) (GtkRadioMenuItem *radio_menu_item);
 
@@ -66,7 +70,7 @@ struct _GtkRadioMenuItemClass
 
 
 GType      gtk_radio_menu_item_get_type	         (void) G_GNUC_CONST;
-							
+
 GtkWidget* gtk_radio_menu_item_new                           (GSList           *group);
 GtkWidget* gtk_radio_menu_item_new_with_label                (GSList           *group,
 							      const gchar      *label);
@@ -81,10 +85,6 @@ GSList*    gtk_radio_menu_item_get_group                     (GtkRadioMenuItem *
 void       gtk_radio_menu_item_set_group                     (GtkRadioMenuItem *radio_menu_item,
 							      GSList           *group);
 
-#ifndef GTK_DISABLE_DEPRECATED
-#define gtk_radio_menu_item_group gtk_radio_menu_item_get_group
-#endif
-
 G_END_DECLS
 
 #endif /* __GTK_RADIO_MENU_ITEM_H__ */
