@@ -47,6 +47,34 @@
 #include "gtkprivate.h"
 #include "gtkalias.h"
 
+
+/**
+ * SECTION:gtkrecentchoosermenu
+ * @Short_description: Displays recently used files in a menu
+ * @Title: GtkRecentChooserMenu
+ * @See_also:#GtkRecentChooser
+ *
+ * #GtkRecentChooserMenu is a widget suitable for displaying recently used files
+ * inside a menu.  It can be used to set a sub-menu of a #GtkMenuItem using
+ * gtk_menu_item_set_submenu(), or as the menu of a #GtkMenuToolButton.
+ *
+ * Note that #GtkRecentChooserMenu does not have any methods of its own. Instead,
+ * you should use the functions that work on a #GtkRecentChooser.
+ *
+ * Note also that #GtkRecentChooserMenu does not support multiple filters, as it
+ * has no way to let the user choose between them as the #GtkRecentChooserWidget
+ * and #GtkRecentChooserDialog widgets do. Thus using gtk_recent_chooser_add_filter()
+ * on a #GtkRecentChooserMenu widget will yield the same effects as using
+ * gtk_recent_chooser_set_filter(), replacing any currently set filter
+ * with the supplied filter; gtk_recent_chooser_remove_filter() will remove
+ * any currently set #GtkRecentFilter object and will unset the current filter;
+ * gtk_recent_chooser_list_filters() will return a list containing a single
+ * #GtkRecentFilter object.
+ *
+ * Recently used files are supported since GTK+ 2.10.
+ */
+
+
 struct _GtkRecentChooserMenuPrivate
 {
   /* the recent manager object */
@@ -161,10 +189,10 @@ static void     manager_changed_cb (GtkRecentManager *manager,
 				    gpointer          user_data);
 
 static void gtk_recent_chooser_activatable_iface_init (GtkActivatableIface  *iface);
-static void gtk_recent_chooser_activatable_update     (GtkActivatable       *activatable,
+static void gtk_recent_chooser_update                 (GtkActivatable       *activatable,
 						       GtkAction            *action,
 						       const gchar          *property_name);
-static void gtk_recent_chooser_activatable_reset      (GtkActivatable       *activatable,
+static void gtk_recent_chooser_sync_action_properties (GtkActivatable       *activatable,
 						       GtkAction            *action);
 
 G_DEFINE_TYPE_WITH_CODE (GtkRecentChooserMenu,
@@ -193,12 +221,11 @@ gtk_recent_chooser_iface_init (GtkRecentChooserIface *iface)
   iface->list_filters = gtk_recent_chooser_menu_list_filters;
 }
 
-static void 
-gtk_recent_chooser_activatable_iface_init (GtkActivatableIface  *iface)
-
-{  
-  iface->update = gtk_recent_chooser_activatable_update;
-  iface->reset = gtk_recent_chooser_activatable_reset;
+static void
+gtk_recent_chooser_activatable_iface_init (GtkActivatableIface *iface)
+{
+  iface->update = gtk_recent_chooser_update;
+  iface->sync_action_properties = gtk_recent_chooser_sync_action_properties;
 }
 
 static void
@@ -834,6 +861,9 @@ gtk_recent_chooser_menu_create_item (GtkRecentChooserMenu *menu,
 
   g_free (text);
 
+  gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (item),
+                                             TRUE);
+
   /* ellipsize the menu item label, in case the recent document
    * display name is huge.
    */
@@ -970,6 +1000,8 @@ idle_populate_func (gpointer data)
 
 	  return FALSE;
 	}
+      else
+        gtk_widget_hide (pdata->placeholder);
       
       pdata->n_items = g_list_length (pdata->items);
       pdata->loaded_items = 0;
@@ -1056,9 +1088,8 @@ gtk_recent_chooser_menu_populate (GtkRecentChooserMenu *menu)
 
   priv->icon_size = get_icon_size_for_widget (GTK_WIDGET (menu));
   
-  /* remove our menu items first and hide the placeholder */
+  /* remove our menu items first */
   gtk_recent_chooser_menu_dispose_items (menu);
-  gtk_widget_hide (priv->placeholder);
   
   priv->populate_id = gdk_threads_add_idle_full (G_PRIORITY_HIGH_IDLE + 30,
   					         idle_populate_func,
@@ -1169,24 +1200,24 @@ gtk_recent_chooser_menu_set_show_tips (GtkRecentChooserMenu *menu,
   gtk_container_foreach (GTK_CONTAINER (menu), foreach_set_shot_tips, menu);
 }
 
-static void 
-gtk_recent_chooser_activatable_update (GtkActivatable       *activatable,
-				       GtkAction            *action,
-				       const gchar          *property_name)
+static void
+gtk_recent_chooser_update (GtkActivatable *activatable,
+			   GtkAction      *action,
+			   const gchar    *property_name)
 {
   if (strcmp (property_name, "sensitive") == 0)
     gtk_widget_set_sensitive (GTK_WIDGET (activatable), gtk_action_is_sensitive (action));
 
-  _gtk_recent_chooser_activatable_update (activatable, action, property_name);
+  _gtk_recent_chooser_update (activatable, action, property_name);
 }
 
-static void 
-gtk_recent_chooser_activatable_reset (GtkActivatable       *activatable,
-				      GtkAction            *action)
+static void
+gtk_recent_chooser_sync_action_properties (GtkActivatable *activatable,
+				           GtkAction      *action)
 {
   gtk_widget_set_sensitive (GTK_WIDGET (activatable), gtk_action_is_sensitive (action));
 
-  _gtk_recent_chooser_activatable_reset (activatable, action);
+  _gtk_recent_chooser_sync_action_properties (activatable, action);
 }
 
 
