@@ -156,6 +156,7 @@ gtk_debug_flag_get_type (void)
             { GTK_DEBUG_ICONTHEME, "GTK_DEBUG_ICONTHEME", "icontheme" },
             { GTK_DEBUG_PRINTING, "GTK_DEBUG_PRINTING", "printing" },
             { GTK_DEBUG_BUILDER, "GTK_DEBUG_BUILDER", "builder" },
+            { GTK_DEBUG_EXTENDED_LAYOUT, "GTK_DEBUG_EXTENDED_LAYOUT", "extended-layout" },
             { 0, NULL, NULL }
         };
         etype = g_flags_register_static (g_intern_static_string ("GtkDebugFlag"), values);
@@ -355,22 +356,6 @@ gtk_button_box_style_get_type (void)
     return etype;
 }
 
-GType
-gtk_curve_type_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_CURVE_TYPE_LINEAR, "GTK_CURVE_TYPE_LINEAR", "linear" },
-            { GTK_CURVE_TYPE_SPLINE, "GTK_CURVE_TYPE_SPLINE", "spline" },
-            { GTK_CURVE_TYPE_FREE, "GTK_CURVE_TYPE_FREE", "free" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkCurveType"), values);
-    }
-    return etype;
-}
-
 GType
 gtk_delete_type_get_type (void)
 {
@@ -464,23 +449,6 @@ gtk_sensitivity_type_get_type (void)
     return etype;
 }
 
-GType
-gtk_side_type_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_SIDE_TOP, "GTK_SIDE_TOP", "top" },
-            { GTK_SIDE_BOTTOM, "GTK_SIDE_BOTTOM", "bottom" },
-            { GTK_SIDE_LEFT, "GTK_SIDE_LEFT", "left" },
-            { GTK_SIDE_RIGHT, "GTK_SIDE_RIGHT", "right" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkSideType"), values);
-    }
-    return etype;
-}
-
 GType
 gtk_text_direction_get_type (void)
 {
@@ -514,25 +482,6 @@ gtk_justification_get_type (void)
     return etype;
 }
 
-GType
-gtk_match_type_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_MATCH_ALL, "GTK_MATCH_ALL", "all" },
-            { GTK_MATCH_ALL_TAIL, "GTK_MATCH_ALL_TAIL", "all-tail" },
-            { GTK_MATCH_HEAD, "GTK_MATCH_HEAD", "head" },
-            { GTK_MATCH_TAIL, "GTK_MATCH_TAIL", "tail" },
-            { GTK_MATCH_EXACT, "GTK_MATCH_EXACT", "exact" },
-            { GTK_MATCH_LAST, "GTK_MATCH_LAST", "last" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkMatchType"), values);
-    }
-    return etype;
-}
-
 GType
 gtk_menu_direction_type_get_type (void)
 {
@@ -741,21 +690,6 @@ gtk_position_type_get_type (void)
     return etype;
 }
 
-GType
-gtk_preview_type_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_PREVIEW_COLOR, "GTK_PREVIEW_COLOR", "color" },
-            { GTK_PREVIEW_GRAYSCALE, "GTK_PREVIEW_GRAYSCALE", "grayscale" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkPreviewType"), values);
-    }
-    return etype;
-}
-
 GType
 gtk_relief_style_get_type (void)
 {
@@ -788,25 +722,6 @@ gtk_resize_mode_get_type (void)
     return etype;
 }
 
-GType
-gtk_signal_run_type_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GFlagsValue values[] = {
-            { GTK_RUN_FIRST, "GTK_RUN_FIRST", "first" },
-            { GTK_RUN_LAST, "GTK_RUN_LAST", "last" },
-            { GTK_RUN_BOTH, "GTK_RUN_BOTH", "both" },
-            { GTK_RUN_NO_RECURSE, "GTK_RUN_NO_RECURSE", "no-recurse" },
-            { GTK_RUN_ACTION, "GTK_RUN_ACTION", "action" },
-            { GTK_RUN_NO_HOOKS, "GTK_RUN_NO_HOOKS", "no-hooks" },
-            { 0, NULL, NULL }
-        };
-        etype = g_flags_register_static (g_intern_static_string ("GtkSignalRunType"), values);
-    }
-    return etype;
-}
-
 GType
 gtk_scroll_type_get_type (void)
 {
@@ -1430,7 +1345,6 @@ gtk_object_flags_get_type (void)
     if (G_UNLIKELY(etype == 0)) {
         static const GFlagsValue values[] = {
             { GTK_IN_DESTRUCTION, "GTK_IN_DESTRUCTION", "in-destruction" },
-            { GTK_FLOATING, "GTK_FLOATING", "floating" },
             { GTK_RESERVED_1, "GTK_RESERVED_1", "reserved-1" },
             { GTK_RESERVED_2, "GTK_RESERVED_2", "reserved-2" },
             { 0, NULL, NULL }
@@ -1440,24 +1354,6 @@ gtk_object_flags_get_type (void)
     return etype;
 }
 
-GType
-gtk_arg_flags_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GFlagsValue values[] = {
-            { GTK_ARG_READABLE, "GTK_ARG_READABLE", "readable" },
-            { GTK_ARG_WRITABLE, "GTK_ARG_WRITABLE", "writable" },
-            { GTK_ARG_CONSTRUCT, "GTK_ARG_CONSTRUCT", "construct" },
-            { GTK_ARG_CONSTRUCT_ONLY, "GTK_ARG_CONSTRUCT_ONLY", "construct-only" },
-            { GTK_ARG_CHILD_ARG, "GTK_ARG_CHILD_ARG", "child-arg" },
-            { 0, NULL, NULL }
-        };
-        etype = g_flags_register_static (g_intern_static_string ("GtkArgFlags"), values);
-    }
-    return etype;
-}
-
 /* enumerations from "gtkprintoperation.h" */
 GType
 gtk_print_status_get_type (void)
@@ -1532,6 +1428,35 @@ gtk_print_error_get_type (void)
     return etype;
 }
 
+/* enumerations from "gtkprivate.h" */
+GType
+gtk_private_flags_get_type (void)
+{
+    static GType etype = 0;
+    if (G_UNLIKELY(etype == 0)) {
+        static const GFlagsValue values[] = {
+            { PRIVATE_GTK_USER_STYLE, "PRIVATE_GTK_USER_STYLE", "user-style" },
+            { PRIVATE_GTK_RESIZE_PENDING, "PRIVATE_GTK_RESIZE_PENDING", "resize-pending" },
+            { PRIVATE_GTK_HAS_POINTER, "PRIVATE_GTK_HAS_POINTER", "has-pointer" },
+            { PRIVATE_GTK_SHADOWED, "PRIVATE_GTK_SHADOWED", "shadowed" },
+            { PRIVATE_GTK_HAS_SHAPE_MASK, "PRIVATE_GTK_HAS_SHAPE_MASK", "has-shape-mask" },
+            { PRIVATE_GTK_IN_REPARENT, "PRIVATE_GTK_IN_REPARENT", "in-reparent" },
+            { PRIVATE_GTK_DIRECTION_SET, "PRIVATE_GTK_DIRECTION_SET", "direction-set" },
+            { PRIVATE_GTK_DIRECTION_LTR, "PRIVATE_GTK_DIRECTION_LTR", "direction-ltr" },
+            { PRIVATE_GTK_ANCHORED, "PRIVATE_GTK_ANCHORED", "anchored" },
+            { PRIVATE_GTK_CHILD_VISIBLE, "PRIVATE_GTK_CHILD_VISIBLE", "child-visible" },
+            { PRIVATE_GTK_REDRAW_ON_ALLOC, "PRIVATE_GTK_REDRAW_ON_ALLOC", "redraw-on-alloc" },
+            { PRIVATE_GTK_ALLOC_NEEDED, "PRIVATE_GTK_ALLOC_NEEDED", "alloc-needed" },
+            { PRIVATE_GTK_REQUEST_NEEDED, "PRIVATE_GTK_REQUEST_NEEDED", "request-needed" },
+            { PRIVATE_GTK_WIDTH_REQUEST_NEEDED, "PRIVATE_GTK_WIDTH_REQUEST_NEEDED", "width-request-needed" },
+            { PRIVATE_GTK_HEIGHT_REQUEST_NEEDED, "PRIVATE_GTK_HEIGHT_REQUEST_NEEDED", "height-request-needed" },
+            { 0, NULL, NULL }
+        };
+        etype = g_flags_register_static (g_intern_static_string ("GtkPrivateFlags"), values);
+    }
+    return etype;
+}
+
 /* enumerations from "gtkprogressbar.h" */
 GType
 gtk_progress_bar_style_get_type (void)
@@ -1819,24 +1744,6 @@ gtk_text_window_type_get_type (void)
 }
 
 /* enumerations from "gtktoolbar.h" */
-GType
-gtk_toolbar_child_type_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_TOOLBAR_CHILD_SPACE, "GTK_TOOLBAR_CHILD_SPACE", "space" },
-            { GTK_TOOLBAR_CHILD_BUTTON, "GTK_TOOLBAR_CHILD_BUTTON", "button" },
-            { GTK_TOOLBAR_CHILD_TOGGLEBUTTON, "GTK_TOOLBAR_CHILD_TOGGLEBUTTON", "togglebutton" },
-            { GTK_TOOLBAR_CHILD_RADIOBUTTON, "GTK_TOOLBAR_CHILD_RADIOBUTTON", "radiobutton" },
-            { GTK_TOOLBAR_CHILD_WIDGET, "GTK_TOOLBAR_CHILD_WIDGET", "widget" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkToolbarChildType"), values);
-    }
-    return etype;
-}
-
 GType
 gtk_toolbar_space_style_get_type (void)
 {
@@ -1970,6 +1877,7 @@ gtk_widget_flags_get_type (void)
             { GTK_RECEIVES_DEFAULT, "GTK_RECEIVES_DEFAULT", "receives-default" },
             { GTK_DOUBLE_BUFFERED, "GTK_DOUBLE_BUFFERED", "double-buffered" },
             { GTK_NO_SHOW_ALL, "GTK_NO_SHOW_ALL", "no-show-all" },
+            { GTK_MULTIDEVICE, "GTK_MULTIDEVICE", "multidevice" },
             { 0, NULL, NULL }
         };
         etype = g_flags_register_static (g_intern_static_string ("GtkWidgetFlags"), values);
@@ -1992,145 +1900,6 @@ gtk_widget_help_type_get_type (void)
     return etype;
 }
 
-/* enumerations from "gtktree.h" */
-GType
-gtk_tree_view_mode_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_TREE_VIEW_LINE, "GTK_TREE_VIEW_LINE", "line" },
-            { GTK_TREE_VIEW_ITEM, "GTK_TREE_VIEW_ITEM", "item" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkTreeViewMode"), values);
-    }
-    return etype;
-}
-
-/* enumerations from "gtkclist.h" */
-GType
-gtk_cell_type_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_CELL_EMPTY, "GTK_CELL_EMPTY", "empty" },
-            { GTK_CELL_TEXT, "GTK_CELL_TEXT", "text" },
-            { GTK_CELL_PIXMAP, "GTK_CELL_PIXMAP", "pixmap" },
-            { GTK_CELL_PIXTEXT, "GTK_CELL_PIXTEXT", "pixtext" },
-            { GTK_CELL_WIDGET, "GTK_CELL_WIDGET", "widget" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkCellType"), values);
-    }
-    return etype;
-}
-
-GType
-gtk_clist_drag_pos_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_CLIST_DRAG_NONE, "GTK_CLIST_DRAG_NONE", "none" },
-            { GTK_CLIST_DRAG_BEFORE, "GTK_CLIST_DRAG_BEFORE", "before" },
-            { GTK_CLIST_DRAG_INTO, "GTK_CLIST_DRAG_INTO", "into" },
-            { GTK_CLIST_DRAG_AFTER, "GTK_CLIST_DRAG_AFTER", "after" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkCListDragPos"), values);
-    }
-    return etype;
-}
-
-GType
-gtk_button_action_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GFlagsValue values[] = {
-            { GTK_BUTTON_IGNORED, "GTK_BUTTON_IGNORED", "ignored" },
-            { GTK_BUTTON_SELECTS, "GTK_BUTTON_SELECTS", "selects" },
-            { GTK_BUTTON_DRAGS, "GTK_BUTTON_DRAGS", "drags" },
-            { GTK_BUTTON_EXPANDS, "GTK_BUTTON_EXPANDS", "expands" },
-            { 0, NULL, NULL }
-        };
-        etype = g_flags_register_static (g_intern_static_string ("GtkButtonAction"), values);
-    }
-    return etype;
-}
-
-/* enumerations from "gtkctree.h" */
-GType
-gtk_ctree_pos_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_CTREE_POS_BEFORE, "GTK_CTREE_POS_BEFORE", "before" },
-            { GTK_CTREE_POS_AS_CHILD, "GTK_CTREE_POS_AS_CHILD", "as-child" },
-            { GTK_CTREE_POS_AFTER, "GTK_CTREE_POS_AFTER", "after" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkCTreePos"), values);
-    }
-    return etype;
-}
-
-GType
-gtk_ctree_line_style_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_CTREE_LINES_NONE, "GTK_CTREE_LINES_NONE", "none" },
-            { GTK_CTREE_LINES_SOLID, "GTK_CTREE_LINES_SOLID", "solid" },
-            { GTK_CTREE_LINES_DOTTED, "GTK_CTREE_LINES_DOTTED", "dotted" },
-            { GTK_CTREE_LINES_TABBED, "GTK_CTREE_LINES_TABBED", "tabbed" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkCTreeLineStyle"), values);
-    }
-    return etype;
-}
-
-GType
-gtk_ctree_expander_style_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_CTREE_EXPANDER_NONE, "GTK_CTREE_EXPANDER_NONE", "none" },
-            { GTK_CTREE_EXPANDER_SQUARE, "GTK_CTREE_EXPANDER_SQUARE", "square" },
-            { GTK_CTREE_EXPANDER_TRIANGLE, "GTK_CTREE_EXPANDER_TRIANGLE", "triangle" },
-            { GTK_CTREE_EXPANDER_CIRCULAR, "GTK_CTREE_EXPANDER_CIRCULAR", "circular" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkCTreeExpanderStyle"), values);
-    }
-    return etype;
-}
-
-GType
-gtk_ctree_expansion_type_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_CTREE_EXPANSION_EXPAND, "GTK_CTREE_EXPANSION_EXPAND", "expand" },
-            { GTK_CTREE_EXPANSION_EXPAND_RECURSIVE, "GTK_CTREE_EXPANSION_EXPAND_RECURSIVE", "expand-recursive" },
-            { GTK_CTREE_EXPANSION_COLLAPSE, "GTK_CTREE_EXPANSION_COLLAPSE", "collapse" },
-            { GTK_CTREE_EXPANSION_COLLAPSE_RECURSIVE, "GTK_CTREE_EXPANSION_COLLAPSE_RECURSIVE", "collapse-recursive" },
-            { GTK_CTREE_EXPANSION_TOGGLE, "GTK_CTREE_EXPANSION_TOGGLE", "toggle" },
-            { GTK_CTREE_EXPANSION_TOGGLE_RECURSIVE, "GTK_CTREE_EXPANSION_TOGGLE_RECURSIVE", "toggle-recursive" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkCTreeExpansionType"), values);
-    }
-    return etype;
-}
-
 #define __GTK_TYPE_BUILTINS_C__
 #include "gtkaliasdef.c"
 
