@@ -9,6 +9,8 @@
 *tp++ = gdk_cursor_get_type();
 *tp++ = gdk_cursor_type_get_type();
 *tp++ = gdk_device_get_type();
+*tp++ = gdk_device_manager_get_type();
+*tp++ = gdk_device_type_get_type();
 *tp++ = gdk_display_get_type();
 *tp++ = gdk_display_manager_get_type();
 *tp++ = gdk_drag_action_get_type();
@@ -27,6 +29,7 @@
 *tp++ = gdk_function_get_type();
 *tp++ = gdk_gc_get_type();
 *tp++ = gdk_gc_values_mask_get_type();
+*tp++ = gdk_grab_ownership_get_type();
 *tp++ = gdk_grab_status_get_type();
 *tp++ = gdk_gravity_get_type();
 *tp++ = gdk_image_get_type();
@@ -317,7 +320,6 @@
 *tp++ = gtk_toggle_action_get_type();
 *tp++ = gtk_toggle_button_get_type();
 *tp++ = gtk_toggle_tool_button_get_type();
-*tp++ = gtk_toolbar_child_type_get_type();
 *tp++ = gtk_toolbar_get_type();
 *tp++ = gtk_toolbar_space_style_get_type();
 *tp++ = gtk_toolbar_style_get_type();
